// dimensions de la visualization.
const width = 800;
const height = 400;
const margin = {top:20, right:20, bottom:20, left:20};

// SVG canvas.
const svg = d3.select("#arbre_svg")
  .attr("width", width)
  .attr("height", height);

const g = svg.append("g")
  .attr("transform", `translate(${margin.left},${margin.top})`);

// la forêt d'arbres
let data = [
  {
    id: 1,
    name: "Arbre 1",
    accuracy: 0.9,
    children: [
      {
        name: "x1",
        children: [
          { name: "x2" },
          {
            name: "x3",
            children: [
              { name: "x4" },
              { name: "x5" },
            ],
          },
        ],
      },
    ],
  },
  {
    id: 2,
    name: "Arbre 2",
    accuracy: 0.4,
    children: [
      {
        name: "x6",
        children: [
          {
            name: "x3",
            children: [
              { name: "x1" },
              { name: "x8" },
            ],
          },
          {
            name: "x7",
            children: [
              { name: "x2" },
              { name: "x4" },
            ],
          },
        ],
      },
    ],
  },
  {
    id: 3,
    name: "Arbre 3",
    accuracy: 0.7,
    children: [
      {
        name: "x8",
        children: [
          {
            name: "x4",
            children: [
              { name: "x1" },
              { name: "x3" },
            ],
          },
          {
            name: "x5",
            children: [
              { name: "x4" },
              { name: "x2" },
            ],
          },
        ],
      },
    ],
  },
  {
    id: 4,
    name: "Arbre 4",
    accuracy: 0.8,
    children: [
      {
        name: "x2",
        children: [
          {
            name: "x1",
            children: [
              { name: "x4" },
              { name: "x3" },
            ],
          },
          { name: "x8" },
        ],
      },
    ],
  },
];

// Trier les arbres par précision dans l'ordre décroissant
data.sort((a, b) => b.accuracy - a.accuracy);

// arbre de disposition
const treeLayout = d3.cluster().size([2 * Math.PI, 0.3 * (width - margin.left - margin.right) / data.length]);

// fonction pour générer les arbres hiérarchiques à partir des données
function generateTree(treeData, treeId) {
  const root = d3.hierarchy(treeData);
  root.each((node, i) => {
    node.id = `${treeId}-${i}`;
  });
  return treeLayout(root);
}

data = data.map((treeData, i) => generateTree(treeData, i + 1));

let xOffset = 50;
let maxHeight = 300;

function toggleSameNodes(d) {
  // Trouver tous les nœuds portant le même nom dans tous les arbres
  const sameNodes = data.flatMap(tree => tree.descendants())
    .filter(node => node.data.name === d.data.name);

  // Basculer la classe "toggled" sur les nœuds trouvés
  svg.selectAll(".node")
    .filter(node => sameNodes.includes(node))
    .classed("toggled", sameNodes[0].data.toggled ? false : true);
}


data.forEach((tree, index) => {
  const treeGroup = g.append("g")
    .attr("transform", `translate(${xOffset + 0.3 * (width - margin.left - margin.right) / data.length}, ${height / 2})`);

  // Ajouter un cercle englobant pour chaque arbre
	treeGroup.append("circle")
    .attr("r", 0.4 * (width - margin.left - margin.right) / data.length)
    .attr("cx", 0)
    .attr("cy", 0)
    .attr("fill", "none")
    .attr("stroke", "black");

  const treeWidths = tree.descendants().map(d => d.x);
  const maxTreeWidth = Math.max(...treeWidths);
  const treeHeights = tree.descendants().map(d => d.y);
  const maxTreeHeight = Math.max(...treeHeights);
	

  // Dessiner les nœuds
  const nodes = treeGroup
    .selectAll(".node")
    .data(tree.descendants())
    .enter()
    .append("g")
      .attr("class", "node")
      .attr("transform", (d) => `rotate(${d.x * 180 / Math.PI - 90}) translate(${d.y}, 0)`)
      .attr("class", function(d) {
    if (!d.parent && d.data.accuracy >= 0.8) {
      return "node high-accuracy";
    } else if (!d.parent && d.data.accuracy >= 0.6) {
      return "node medium-accuracy";
    } else if (!d.parent) {
      return "node low-accuracy";
    } else {
      return "node black";
    }
  })
    .on("mouseover", function(d) {
      tooltip.transition()
        .duration(200)
        .style("opacity", .9);
      tooltip.html(d.data.name)
        .style("left", (d3.event.pageX) + "px")
        .style("top", (d3.event.pageY - 28) + "px");
    })
    .on("mouseout", function(d) {
      tooltip.transition()
        .duration(500)
        .style("opacity", 0);
    })
    .on("click", function(d) {
      const node = d3.select(this);
      const isToggled = node.classed("toggled");

      // Remove the "toggled" class from all nodes in all trees
      svg.selectAll(".node").classed("toggled", false);

      // Toggle the "toggled" class on the clicked node and same nodes
      node.classed("toggled", !isToggled);
      toggleSameNodes(d);
    });
  
  
  
nodes.append("circle")
  .attr("r", function(d) {
    if (d.data.accuracy >= 0.8) {
      return 20;
    } else if (d.data.accuracy >= 0.6) {
      return 15;
    } else {
      return 10;
    }
  })
  .attr("fill", function(d) {
    if (d.data.accuracy >= 0.8) {
      return "red";
    } else if (d.data.accuracy >= 0.6) {
      return "blue";
    } else {
      return "gray";
    }
  })
  .attr("data-name", d => d.data.name);


  // Ajouter une info-bulle pour afficher le nom du nœud
const tooltip = d3.select("body").append("div")
  .attr("class", "tooltip")
  .style("opacity", 0);

// Dessiner les liens
const links = treeGroup.selectAll(".link")
  .data(tree.links())
  .enter()
  .append("path")
    .attr("class", "link")
    .attr("d", d3.linkRadial()
      .angle(d => d.x)
      .radius(d => d.y));
  
  // Calculate the new xOffset for the next tree
	xOffset += 0.3 * (width - margin.left - margin.right) + 30;


  // Calculate the maxHeight
  maxHeight = Math.max(maxHeight, 2 * maxTreeHeight);
});

// Adjust the dimensions of the SVG canvas if necessary
const totalWidth = xOffset + margin.right;
const totalHeight = maxHeight + margin.top + margin.bottom;
if (totalWidth > width) {
  svg.attr("width", totalWidth);
}

if (totalHeight > height) {
  svg.attr("height", to